@@ -2,25 +2,18 @@ from parse import parse
 
 arrays = ['pkgname', 'arch', 'license', 'groups', 'options',
           'conflicts', 'provides', 'replaces',
-          'source', 'noextract', 'backup', 'validpgpkeys', 'md5sums',
+          'source', 'noextract', 'backup', 'validpgpkeys', 'b2sums', 'md5sums',
           'sha1sums', 'sha224sums', 'sha256sums', 'sha384sums', 'sha512sums',
           'depends', 'makedepends', 'checkdepends', 'optdepends',
           ]
 
 
-def remove_empty_values(values):
-    return [v for v in values if v.strip()]
-
-
 ##
 # Check if SRCINFO key is an array
 #
 def is_array(key):
     if key in arrays:
         return True
-    for array in arrays:
-        if key.startswith(array):
-            return True
     return False
 
 
@@ -29,12 +22,11 @@ def is_array(key):
 #
 def list_insert(target, key, value):
     if key not in target:
-            target[key] = []
+        target[key] = []
 
     # Don't append value if already in the list
-    if value not in target[key]:
-        if value:
-            target[key].append(value)
+    if value and value not in target[key]:
+        target[key].append(value)
 
 
 ##
@@ -42,21 +34,22 @@ def list_insert(target, key, value):
 #
 def extract_var(string):
     line = string.lstrip()
+    if line.startswith('#'):
+        return (None, None, None)
+
     key = None
     value = None
     errors = []
 
-    parsed = parse('{} ={}', line)
+    def value(string):
+        return string.strip()
+    value.pattern = r'.*'
+
+    parsed = parse('{:^}={:value}', line, dict(value=value))
     if parsed:
         (key, value) = parsed
-        key = key.strip()
-        value = value.strip()
-
-    elif line.startswith('#'):
-        pass
-
     else:
-        errors.append('failed to parse line: "{}"'.format(line))
+        errors.append('failed to parse line: {!r}'.format(line))
 
     return (key, value, errors)
 
@@ -77,19 +70,33 @@ def parse_srcinfo(source):
         if not line.strip():
             continue
 
-        if line.startswith('pkgname'):
-            pkgname = line.split('pkgname =')[1].strip()
-            info = srcinfo['packages'][pkgname] = {}
-            continue
-
         (key, value, err) = extract_var(line)
+
         if err:
             errors.append({
                 'line': index,
                 'error': err
             })
 
-        if not key:
+        if key == 'pkgbase':
+            if 'pkgbase' in srcinfo:
+                errors.append({
+                    'line': index,
+                    'error': "pkgbase declared more than once",
+                })
+
+            elif info != srcinfo:
+                errors.append({
+                    'line': index,
+                    'error': "pkgbase declared after pkgname",
+                })
+
+        if key == 'pkgname':
+            pkgname = value
+            info = srcinfo['packages'][pkgname] = {}
+            continue
+
+        elif not key:
             continue
 
         if is_array(key):
